VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CChildForms"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
  Option Explicit
  ' demo project showing how to raise events from objects contained
  ' in a collection or array without creating circular references
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.
  
  
  ' this class is built around the VB collection object
  Dim m_oCol As Collection
  
  ' indexes of the events that will be raised by the child objects
  Public Enum EVENT_INDEXES
    cf_ChildActivated
    cf_ChildDeActivated
    cf_ChildGotFocus
    cf_ChildLostFocus
    cf_ChildLoaded
    cf_ChildUnLoaded
    cf_ChildButtonClicked
  End Enum
  
  ' the events that we will raise in the client
  Event ChildActivated(ByVal nIndex&, ByVal sKey$)
  Event ChildDeActivated(ByVal nIndex&, ByVal sKey$)
  Event ChildGotFocus(ByVal nIndex&, ByVal sKey$)
  Event ChildLostFocus(ByVal nIndex&, ByVal sKey$)
  Event ChildLoaded(ByVal nIndex&, ByVal sKey$)
  Event ChildUnLoaded(ByVal nIndex&, ByVal sKey$)
  Event ChildButtonClicked(ByVal nIndex&, ByVal sKey$)
  
Private Sub Class_Initialize()
  ' create an instance of the collection
  Set m_oCol = New Collection

  ' add a "dumb" (unreferenced) pointer to the array in the MValidatePointer BAS module
  MValidatePointer.AddToLookupList ObjPtr(Me)
End Sub

Private Sub Class_Terminate()
  ' destroy the collection
  Set m_oCol = Nothing
  
  ' remove the pointer we stored in the class initialize event
  MValidatePointer.RemoveFromLookupList ObjPtr(Me)
End Sub

Public Sub Add()
  ' our Add method.  this method adds a new instance of a child form to
  ' the collection after initializing the child
  
  Dim oChildFrm As frmMDIChild, sItemsCount$
  
  ' get the number of items in the collection so that we can give this one a unique key
  sItemsCount = CStr(m_oCol.Count + 1)
    
  ' create a new instance of the child form
  Set oChildFrm = New frmMDIChild
    
  ' initialize the child....
  With oChildFrm
    ' pass a dumb pointer to this collection object so it can be stored in the child object
    .CollectionPointer = ObjPtr(Me)
    
    ' set the key property in the child
    .Key = oChildFrm.Name & sItemsCount
    
    ' change the caption of the child to reflect who it is
    .Caption = .Caption & " #" & sItemsCount
    
    ' display the child
    .Show
    
    ' add the child to the collection
    m_oCol.Add oChildFrm, .Key
  End With
  
  ' destroy this local reference to the child
  Set oChildFrm = Nothing
  
End Sub

Friend Sub FireEvent(ByVal nEvent As EVENT_INDEXES, ByVal sKey$, ByVal lpObject&)
  ' this method is called from the children of this collection to fire events in
  ' the collection client
  
  Dim i&, nIndex&
  
  ' determine the index of the calling object.  if your collection is going to
  ' hold large numbers of items, you may want to reevaluate this part of the code
  For i = 1 To m_oCol.Count
    If ObjPtr(m_oCol(i)) = lpObject Then
      nIndex = i
      Exit For
    End If
  Next
  
  If nIndex > 0 Then
    ' raise the desired event
    Select Case nEvent
      Case cf_ChildActivated: RaiseEvent ChildActivated(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildDeActivated: RaiseEvent ChildDeActivated(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildGotFocus: RaiseEvent ChildGotFocus(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildLostFocus: RaiseEvent ChildLostFocus(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildLoaded: RaiseEvent ChildLoaded(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildUnLoaded: RaiseEvent ChildUnLoaded(ByVal nIndex&, ByVal sKey$)
      Case cf_ChildButtonClicked: RaiseEvent ChildButtonClicked(ByVal nIndex&, ByVal sKey$)
    End Select
  End If
  
End Sub
